logmode +addin

!dogui = 1		'use dialog
%select = ""		'selection equation specification
%response = ""	'response equation specification
!method = 1		'estimation method 1 = "Two step", 2 = ML
%coefs = ""		'matrix to save coefs into (leave blank to not save)
%covar = ""		'matrix to save covariance matrix into (leave blank to not save)
%smpl = @pagesmpl  'sample over which to estimate
!doanal = 1		'use analytic derivatives for ML estimation
!debug = 0		'turn on debugging (both EViews and R-Log)



include ".\heckman2step.prg"
include ".\heckmanml.prg"
include ".\tablemake.prg"

setmaxerrs 1
if !debug = 1 then
	logmode +debug
endif

'get original sample string
%origsmpl = @pagesmpl

'parse arguments
if @len(%args) > 0 then
	'find @ symbol
	!pos = @instr(%args," @ ")
	%select = @left(%args,!pos)
	%response = @mid(%args,!pos+3)	
	
	if @len(%select) > 0 and @len(%response) > 0 then
		!dogui = 0
	endif
	
endif


'parse options
if @len(@option(1)) > 0 then
	%coefs = @equaloption("coefmat")
	%covar = @equaloption("covarmat")
	!method = @hasoption("ml") + 1
	if @hasoption("prompt") = 1 then
		!dogui = 1
	endif
	!doanal = @hasoption("d")
endif

%methods = """Two-step"" ""Maximum Likelihood""" 
if !dogui = 1 then
	'put up dialog
	!return = @uidialog("caption", "Heckman Selection Model", _
 						"edit", %select, "Selection Equation (enter a dependent variable followed by a list of regressors)", 500, _
 						"edit", %response, "Response Equation (enter a dependent variable followed by a list of regressors)", 500, _
 						"list", !method, "Estimation method", %methods, _
 						"colbreak", _
 						"edit",%smpl,"Enter an estimation sample", 500, _
 						"edit", %coefs, "Output Coefficients (optional)", _
 						"edit", %covar, "Output Covariance (optional)", _
 						"text", "", _
 						"check", !doanal, "Use analytic derivatives in ML" _
 						)
 	if !return = -1 then
 		stop
 	endif
endif
 
 'set sample to specified estimation sample
 smpl {%smpl}

 
 'check inputted equations are non-null
 if @wcount(%select) < 2 then
 	seterr "Invalid selection equation"
 endif
 if @wcount(%response) < 2 then
 	seterr "Invalid response equation"
 endif
 

'check output names are valid and aren't already objects
if @len(%coefs) then
	if @isvalidname(%coefs) = 0 then
		seterr "Invalid coefficient output name"
	endif
	if @isobject(%coefs) then
		%msg = %coefs + " already exists.  Overwrite?"
		!result = @uiprompt(%msg,"YN")
		if !result = 2 then
			stop
		endif
		d {%coefs}
	endif
endif
if @len(%covar) then
	if @isvalidname(%covar) = 0 then
		seterr "Invalid covariance output name"
	endif
	if @isobject(%covar) then
		%msg = %covar+ " already exists.  Overwrite?"
		!result = @uiprompt(%msg,"YN")
		if !result = 2 then
			stop
		endif
		d {%covar}
	endif
endif



'make temporary matrices for storing coefficients
!sellen = @wcount(%select)-1
!reslen = @wcount(%response)-1

%bn = @getnextname("_B")
%cn = @getnextname("_C")
%rhon = @getnextname("_rho")
%sign = @getnextname("_sig")
%covn = @getnextname("_cov")
%alln = @getnextname("_allcoefs")

coef(!sellen) {%bn}
coef(!reslen) {%cn}
coef(1) {%rhon}
coef(1) {%sign}
!rows = @wcount(%select) - 1 + @wcount(%response)
if !method = 2 then
	matrix(1,1) {%covn}
else
	matrix(!rows,!rows) {%covn}
endif
!snobs = 0
!onobs = 0
!df = 0


'call two-step estimator
call twostep(%select, %response, {%bn}, {%cn}, {%rhon}, {%sign}, {%covn}, !snobs, !onobs, !df)


'call ML estimator
if !method = 2 then
	if {%rhon}(1,1) > 1 then
		{%rhon} = 1
	endif
	if {%rhon}(1,1) < -1 then
		{%rhon} = -1
	endif
	call ml(%select, %response, {%bn}, {%cn}, {%rhon}, {%sign}, {%covn}, !doanal)
endif

'put coefs into one big coefficient matrix.
!rows = @rows({%bn}) + @rows({%cn}) + 2
coef(!rows) {%alln}
matplace({%alln},{%bn},1,1)
!row = @rows({%bn}) + 1
matplace({%alln},{%cn},!row,1)
!row = !row + @rows({%cn})
matplace({%alln},{%sign},!row,1)
!row = !row + 1
matplace({%alln},{%rhon},!row,1)

'make output table
%method = @word(%methods,!method)
%tn = @getnextname("_TABLE")
call maketable(%tn, %alln, %covn, %select, %response, %method, %smpl, !snobs, !onobs, !df)

'make display spool
%sn = "results"
if @isobject(%sn) then
	%sn = @getnextname(%sn)
endif

spool {%sn}
{%sn}.append {%tn}
{%sn}.options -tree -borders -titles -comments -margins

show {%sn}

'store results into workfile
if @len(%coefs) > 0 then
	coef {%coefs} = {%alln}
endif
if @len(%covar) > 0 then
	matrix {%covar} = {%covn}
endif


'clean up
if !debug <> 1 then
	d {%tn} {%bn} {%cn} {%rhon} {%sign} {%alln} {%covn}
endif
