'note that the size of cov indicates whether to calculate the covariance matrix or not.  If it is a 1x1 matrix, covariance matrix will not be calculated.
subroutine twostep(string %selectspec, string %responsespec, coef b, coef cc, coef rho, coef sig, matrix cov, scalar !snobs, scalar !onobs, scalar !df)
	local smpl
	%smplser = @getnextname("_smplser")
	series {%smplser} = 1
	
	%seldep = @word(%selectspec,1)
	%selregs = @wmid(%selectspec,2)
	%resregs = @wmid(%responsespec,2)
	!imillsidx = @wcount(%response)
	
	%eq1 = @getnextname("_eq1_")
	%eq2 = @getnextname("_eq2_")
	%imills = @getnextname("_imills")
	%delta = @getnextname("_delta")
	%resid = @getnextname("_resid")
	%xbhat = @getnextname("_xbhat")
	
	%selgrp = @getnextname("_selgrp")
	%resgrp = @getnextname("_resgrp")
	
	equation {%eq1}.binary(d=n) {%selectspec}
	
	' copy starting values to selection equation params
	b = {%eq1}.@coefs
	!snobs = {%eq1}.@regobs
	
	' compute inverse mills ratio
	{%eq1}.fit(i) {%xbhat}
	series {%imills} = @dnorm({%xbhat})/@cnorm({%xbhat})
	
	' compute delta to estimate sig2 (variance)
	series {%delta} = {%imills}*({%imills}+{%xbhat})
	
	' run second step OLS, including the inverse mills ratio
	smpl if {%seldep}=1 and {%smplser} = 1
	equation {%eq2}.ls {%responsespec} {%imills}
	!onobs = {%eq2}.@regobs	
	cc = {%eq2}.@coefs
	
	' construct estimate of correlation rho (note: uses only estimation sample of 2nd step)
	{%eq2}.makeresid {%resid}
	
	' initial estimate of the regression model std err
	sig = @sqrt(@sumsq({%resid})/{%eq2}.@regobs+@mean({%delta})*cc(!imillsidx)^2)
	
	' initial estimate of the correlation between the two equations
	rho = cc(!imillsidx)/sig(1)
	
	'covariances..
	if @rows(cov) > 1 then
		'taken off page 4 of the sampleSelction R paper.
		group {%selgrp} {%selregs}
		group {%resgrp} {%resregs} {%imills}
		call twostepcov({%selgrp}, {%resgrp}, {%delta}, {%eq1}.@coefcov , rho(1), sig(1), cov)
	endif
		
	'clean up
	d(noerr) {%smplser}  {%eq2} {%imills} {%delta} {%eq1} {%resid} {%xbhat} {%resgrp} {%selgrp}
	!df = !snobs - @rows(cc) - @rows(b) - 2
	
endsub

subroutine local twostepcov(group xss, group xos, series delta, sym eq1cov, scalar !rho, scalar !sig, matrix cov)
	local smpl
	smpl if @robs(xss)=xss.@count and @robs(xos)=xos.@count
	matrix xo = @convert(xos)
	matrix xs = @convert(xss)
	sym xoxo  = @inverse(@inner(xo))
	series tempser = 1-!rho*!rho*delta
	vector deltas = @convert(tempser)
	sym deltad = @makediagonal(deltas)
	matrix fmat = @transpose(xo)*deltad*xs
	sym q = !rho^2 * fmat * eq1cov * @transpose(fmat)
	matrix covo = !sig^2 * xoxo * (@transpose(xo)*(deltad)*xo+q) * xoxo
	!row = @rows(eq1cov)+1
	matplace(cov,covo,!row,!row)
	matrix temp = eq1cov
	matplace(cov, temp,1,1)	
endsub
