logmode +addin

!dogui = 1		'use dialog
%select = ""		'selection equation specification
%response = ""	'response equation specification
%select = "inlf c age age^2 faminc kids educ"
%response = "wage c exper exper^2 educ city"
!method = 1		'estimation method 1 = "Two step", 2 = ML
%coefs = ""		'matrix to save coefs into (leave blank to not save)
%covar = ""		'matrix to save covariance matrix into (leave blank to not save)
%smpl = @pagesmpl  'sample over which to estimate
!debug = 1		'turn on debugging (both EViews and R-Log)


setmaxerrs 1
if !debug = 1 then
	logmode +debug
endif

'get original sample string
%origsmpl = @pagesmpl

'parse arguments
if @len(%args) > 0 then
	'find @ symbol
	!pos = @instr(%args," @ ")
	%select = @left(%args,!pos)
	%response = @mid(%args,!pos+3)	
	
	if @len(%select) > 0 and @len(%response) > 0 then
		!dogui = 0
	endif
	
endif


'parse options
if @len(@option(1)) > 0 then
	%coefs = @equaloption("coefmat")
	%covar = @equaloption("covarmat")
	!method = @hasoption("ml") + 1
	if @hasoption("prompt") = 1 then
		!dogui = 1
	endif
endif

%methods = """Two-step"" ""Maximum Likelihood""" 
if !dogui = 1 then
	'put up dialog
	!return = @uidialog("caption", "Heckman Selection Model", _
 						"edit", %select, "Selection Equation (enter a dependent variable followed by a list of regressors)", 500, _
 						"edit", %response, "Response Equation (enter a dependent variable followed by a list of regressors)", 500, _
 						"list", !method, "Estimation method", %methods, _
 						"colbreak", _
 						"edit",%smpl,"Enter an estimation sample", 500, _
 						"edit", %coefs, "Output Coefficients (optional)", _
 						"edit", %covar, "Output Covariance (optional)" _
 						)
 	if !return = -1 then
 		stop
 	endif
endif
 
 'set sample to specified estimation sample
 smpl {%smpl}

 
 'check inputted equations are non-null
 if @wcount(%select) < 2 then
 	seterr "Invalid selection equation"
 endif
 if @wcount(%response) < 2 then
 	seterr "Invalid response equation"
 endif
 
 
 'calculate list of unique variables from the two equation lists
 %names = @wunique(%select +  " " + %response)
 
 'check inputted series are valid
 %gname = @getnextname("_g")
 group {%gname} {%names}
d {%gname}
 

'check output names are valid and aren't already objects
if @len(%coefs) then
	if @isvalidname(%coefs) = 0 then
		seterr "Invalid coefficient output name"
	endif
	if @isobject(%coefs) then
		%msg = %coefs + " already exists.  Overwrite?"
		!result = @uiprompt(%msg,"YN")
		if !result = 2 then
			stop
		endif
		d {%coefs}
	endif
endif
if @len(%covar) then
	if @isvalidname(%covar) = 0 then
		seterr "Invalid covariance output name"
	endif
	if @isobject(%covar) then
		%msg = %covar+ " already exists.  Overwrite?"
		!result = @uiprompt(%msg,"YN")
		if !result = 2 then
			stop
		endif
		d {%covar}
	endif
endif





'build estimation method string
%method = "method="
if !method=1 then 
	%method = %method + """2step"""
else 
	%method = %method + """ml"""
endif

'open up R connection
if !debug = 1 then
	xopen(r)
else 
	xopen(r, nolog)
endif

'open up selection library
xrun "if (sum(.packages(TRUE)==""sampleSelection"")==0) install.packages(""sampleSelection"")"
xrun "library(sampleSelection)"

'remove constant from varible lists, since the R routine adds in the constant by itself
%select = @wdrop(@upper(%select),"C")
%response= @wdrop(@upper(%response),"C")

'send over selection variables and build selection string
%word = @word(%select,1)
xput {%word}
%word = @xputnames
%selectcmd = %word + " ~"
for !i = 2 to @wcount(%select)
	%word = @word(%select,!i)
	xput {%word}
	%word = @xputnames
	%selectcmd = %selectcmd + " " + %word + " +"
next
%selectcmd = @left(%selectcmd ,@len(%selectcmd )-2)  'trim off the final errant " +"


'send over response variables and build response string
%word = @word(%response,1)
xput {%word}
%word = @xputnames
%responsecmd= %word + " ~"
for !i = 2 to @wcount(%response)
	%word = @word(%response,!i)
	xput {%word}
	%word = @xputnames
	%responsecmd= %responsecmd+ " " + %word + " +"
next
%responsecmd= @left(%responsecmd,@len(%responsecmd)-2)  'trim off the final errant " +"


'build estimation command
%cmd = "selec<-selection(" + %selectcmd + "," + %responsecmd + "," + %method +  ")" 

'run command
xrun {%cmd}

'retrieve coefs
if @len(%coefs) = 0 then
	%coefn = @getnextname("_coef")
else
	%coefn = %coefs	
endif
xget(name=%coefn) coef(selec)

'retrieve covar
if @len(%covar) = 0 then
	%covarn = @getnextname("_covar")
else
	%covarn = %covar	
endif
xget(name=%covarn) vcov(selec)

'retrieve number of obs
%n1 = @getnextname("_n1")
xget(name=%n1) selec$param$N1
%nobs = @getnextname("_nobs")
xget(name=%nobs) selec$param$nObs
%df = @getnextname("_df")
xget(name=%df) selec$param$df


'create table
%tn = @getnextname("_TABLE")
!cols = 5
!rows = @rows({%coefn}) + 10
table(!rows,!cols) {%tn}
{%tn}.setwidth(1) 19
{%tn}.setwidth(2:5) 10
{%tn}.setjust(a1:a5) left
!rowcounter=1
%title = "Heckman Selection Model (" + @word(%methods,!method) + ")"
{%tn}(!rowcounter,1) = %title
!rowcounter = !rowcounter + 1
%datestring = "Date: " + @datestr(@now,"MM/DD/YY") + "   Time: " + @datestr(@now,"HH:MI")
{%tn}(!rowcounter,1) = %datestring
!rowcounter=!rowcounter+1
%samplestr = "Sample: " + %smpl
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
%obsstr = "Included selection observations: " + @str({%nobs})
{%tn}(!rowcounter,1) = %obsstr
!rowcounter=!rowcounter+1
%obsstr = "Included response observations: " + @str({%n1})
{%tn}(!rowcounter,1) = %obsstr
!rowcounter=!rowcounter+1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1
!rowcounter = !rowcounter + 1


'--------------Selection Equation
'column headers
{%tn}(!rowcounter,1) = "Selection Equation"
{%tn}.setjust(!rowcounter,1) left
!rowcounter = !rowcounter + 1
%title = "Dependent Variable: " + @upper(@word(%select,1))
{%tn}(!rowcounter,1) = %title
!rowcounter = !rowcounter + 1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

{%tn}(!rowcounter,1) = "Variable"
{%tn}(!rowcounter,2) = "Coefficient"
{%tn}(!rowcounter,3) = "Std. Error"
{%tn}(!rowcounter,4) = "t-Statistic"
{%tn}(!rowcounter,5) = "Prob."
!rowcounter=!rowcounter+1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

'row headers
{%tn}(!rowcounter,1) = "C"
!rowcounter = !rowcounter + 1
!temprow = !rowcounter
for !i = 2 to @wcount(%select)
	{%tn}(!temprow,1) = @upper(@word(%select,!i))
	!temprow = !temprow+1
next

'coefficients etc..
!rowcounter = !rowcounter - 1
for !i=1 to @wcount(%select)   'although first word of select is the dependent variable, we still want to loop through the first word of the results, since it is the intercept.
	!coef = {%coefn}(!i)
	!covar = @sqrt({%covarn}(!i,!i))
	!tstat = !coef/!covar
	!df = {%df}
	!pvalue = @tdist(!tstat,!df)
	{%tn}(!rowcounter ,2) = !coef
	{%tn}(!rowcounter,3) = !covar
	{%tn}(!rowcounter,4) = !tstat
	{%tn}(!rowcounter,5) = !pvalue
	!rowcounter = !rowcounter + 1
next

{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

!rowcounter = !rowcounter + 1

'--------------Response Equation
'column headers
{%tn}(!rowcounter,1) = "Response Equation"
{%tn}.setjust(!rowcounter,1) left
!rowcounter = !rowcounter + 1
%title = "Dependent Variable: " + @upper(@word(%response,1))
{%tn}(!rowcounter,1) = %title
!rowcounter = !rowcounter + 1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

{%tn}(!rowcounter,1) = "Variable"
{%tn}(!rowcounter,2) = "Coefficient"
{%tn}(!rowcounter,3) = "Std. Error"
{%tn}(!rowcounter,4) = "t-Statistic"
{%tn}(!rowcounter,5) = "Prob."
!rowcounter=!rowcounter+1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

'row headers
{%tn}(!rowcounter,1) = "C"
!rowcounter = !rowcounter + 1
!temprow = !rowcounter
for !i = 2 to @wcount(%response)
	{%tn}(!temprow,1) = @upper(@word(%response,!i))
	!temprow = !temprow+1
next

'coefficients etc..
!rowcounter = !rowcounter - 1
!row = @wcount(%select)
for !i=1 to @wcount(%response)
	!j = !i + !row
	!coef = {%coefn}(!j)
	!covar = @sqrt({%covarn}(!j,!j))
	!tstat = !coef/!covar
	!df = {%df}
	!pvalue = @tdist(!tstat,!df)
	{%tn}(!rowcounter ,2) = !coef
	{%tn}(!rowcounter,3) = !covar
	{%tn}(!rowcounter,4) = !tstat
	{%tn}(!rowcounter,5) = !pvalue
	!rowcounter = !rowcounter + 1
next
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1


'---------------------Error Summary

!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "Variable"
{%tn}(!rowcounter,2) = "Coefficient"
{%tn}(!rowcounter,3) = "Std. Error"
{%tn}(!rowcounter,4) = "t-Statistic"
{%tn}(!rowcounter,5) = "Prob."
!rowcounter=!rowcounter+1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

'Inverse Mills Ratio
{%tn}(!rowcounter,1) = "Inv. Mills Ratio"
!row = @rows({%coefn}) - 2
!coef = {%coefn}(!row )
!covar = @sqrt({%covarn}(!row ,!row ))
!tstat = !coef/!covar
!df = {%df}
!pvalue = @tdist(!tstat,!df)
{%tn}(!rowcounter,2) = !coef
{%tn}(!rowcounter,3) = !covar
{%tn}(!rowcounter,4) = !tstat
{%tn}(!rowcounter,5) = !pvalue
!rowcounter = !rowcounter + 1

'Sigma
{%tn}(!rowcounter,1) = "Sigma"
{%tn}(!rowcounter,2) = {%coefn}(!row+1)
!rowcounter = !rowcounter + 1

'Rho
{%tn}(!rowcounter,1) = "Rho"
{%tn}(!rowcounter,2) = {%coefn}(!row+2)
!rowcounter = !rowcounter + 1

{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1


'make display spool
%sn = "results"
if @isobject(%sn) then
	%sn = @getnextname(%sn)
endif

spool {%sn}
{%sn}.append {%tn}
{%sn}.options -tree -borders -titles -comments -margins

show {%sn}
'cleanup
smpl {%origsmpl}
if !debug = 0 then
	d(noerr) {%n1} {%nobs} {%smplser} {%df} {%tn}
	if @len(%coefs) = 0 then
		d {%coefn}
	endif
	
	if @len(%covar) = 0 then
		d {%covarn}
	endif
	xclose
endif
